
/*
 * Copyright (c) 2004, Archie L. Cobbs.
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * * Neither the name "sl2tps" nor the names of its
 *   contributors may be used to endorse or promote products derived
 *   from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Author: Archie Cobbs <archie@freebsd.org>
 */

#ifndef _SL2TPS_CONFIG_H_
#define _SL2TPS_CONFIG_H_

/* User account info */
struct sls_user {
	char			*name;
	char			*password;
	struct in_addr		ip;
};

/* Array of users */
DEFINE_STRUCTS_ARRAY(sls_users_ary, struct sls_user);

/* Array of IP addresses */
DEFINE_STRUCTS_ARRAY(ip_addr_ary, struct in_addr);

/* Configuration object for the sls application */
struct sls_config {
	char			*pidfile;
	struct alog_config	error_log;
	struct in_addr		bind_ip;
	u_int16_t		bind_port;
	struct in_addr		inside_ip;
	struct ip_addr_ary	dns_servers;
	struct ip_addr_ary	nbns_servers;
	struct in_addr		ip_pool_start;
	u_int			ip_pool_size;
	struct sls_users_ary	users;
};

/* Variables */
extern const	struct structs_type sls_config_type;
extern struct	app_config_ctx *sls_config_ctx;
extern const	struct sls_config *const sls_curconf;

/* Functions */
extern int	sls_config_init(struct pevent_ctx *ctx, const char *path);

#endif	/* !_SL2TPS_CONFIG_H_ */

